/*------------------------------ 
/   Quick hack to generate Fontmap entries for ghostscript out of
/   truetype fonts.
/
/ based on: ttf2pt1 (http://www.netspace.net.au/~mark/ttf2pt1/)
/
/ Compile: cc -o ttf-gs ttf-gs.c
/ 
/ Run: ttf-gs <*.ttf>
/ 
/ enjoy, ju
-------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <netinet/in.h>

#include "ttf.h"

static TTF_DIRECTORY *directory;
static TTF_DIR_ENTRY *dir_entry ;
static char *filebuffer;
static TTF_NAME *name_table = NULL;
static TTF_NAME_REC *name_record;
static TTF_HEAD *head_table = NULL;
static TTF_HHEA *hhea_table = NULL;
static TTF_KERN *kern_table = NULL;
static TTF_CMAP *cmap_table = NULL;
static LONGHORMETRIC *hmtx_table = NULL;
static TTF_GLYF *glyf_table;
static BYTE *glyf_start = NULL;
static TTF_MAXP *maxp_table = NULL;
static TTF_POST_HEAD *post_table = NULL;
static USHORT *short_loca_table = NULL;
static ULONG *long_loca_table = NULL;
static int ttf_file, numglyphs, long_offsets, ncurves;

static char name_buffer[2000];
static char *name_fields[8];

static void handle_name ()
{
    int j, k, lang, len, platform;
    char *p, *ptr, *string_area;
    char *nbp = name_buffer;
    int found3 = 0;

    string_area = (char *)name_table + ntohs(name_table->offset);
    name_record = &(name_table->nameRecords);

    for (j=0; j<8; j++) {
        name_fields[j] = NULL;
    }

    for (j=0; j < ntohs(name_table->numberOfNameRecords); j++) {

        platform = ntohs(name_record->platformID);

        if (platform == 3) {

            found3 = 1;
            lang = ntohs(name_record->languageID) & 0xff;
            len = ntohs(name_record->stringLength);
            if (lang == 0 || lang == 9) {
                k = ntohs(name_record->nameID);
                if (k < 8) {
                    name_fields[k] = nbp;

                    p = string_area+ntohs(name_record->stringOffset);
                    for ( k = 0; k < len; k++) {
                        if (p[k] != '\0') {
                            if (p[k] == '(') {
                                *nbp = '[';
                            } else if (p[k] == ')') {
                                *nbp = ']';
                            } else {
                                *nbp = p[k];
                            }
                            nbp ++;
                        }
                    }
                    *nbp = '\0';
                    nbp ++;
                }
            }
        }
        name_record++;
    }

    string_area = (char *)name_table + ntohs(name_table->offset);
    name_record = &(name_table->nameRecords);

    if (!found3) {
        for (j=0; j < ntohs(name_table->numberOfNameRecords); j++) {

            platform = ntohs(name_record->platformID);

            if (platform == 1) {

                found3 = 1;
                lang = ntohs(name_record->languageID) & 0xff;
                len = ntohs(name_record->stringLength);
                if (lang == 0 || lang == 9) {
                    k = ntohs(name_record->nameID);
                    if (k < 8) {
                        name_fields[k] = nbp;

                        p = string_area+ntohs(name_record->stringOffset);
                        for ( k = 0; k < len; k++) {
                            if (p[k] != '\0') {
                                if (p[k] == '(') {
                                    *nbp = '[';
                                } else if (p[k] == ')') {
                                    *nbp = ']';
                                } else {
                                    *nbp = p[k];
                                }
                                nbp ++;
                            }
                        }
                        *nbp = '\0';
                        nbp ++;
                    }
                }
            }
            name_record++;
        }
    }

    if (!found3) {
        fprintf(stderr, "**** Cannot decode font name fields ****\n");
        exit(-1);
    }

    if (name_fields[6] == NULL) {
        name_fields[6] = name_fields[4];
    }

    p = name_fields[6];
    while (*p != '\0') {
        if (!isalnum(*p)) {
            *p = '_';
        }
        p++;
    }
}

main (int argc, char **argv)
{
    int i;
    time_t now;
    float italic_angle;
    struct stat statbuf;

    if (argc != 2) {
        fputs("ttf-gs <ttf-file>\n", stderr);
        exit(-1);
    }

    if (stat(argv[1], &statbuf) == -1) {
        fprintf(stderr, "**** Cannot access %s ****\n", argv[1]);
        exit(-1);
    }

    if ((filebuffer = malloc (statbuf.st_size)) == NULL) {
        fprintf(stderr, "**** Cannot malloc space for file ****\n");
        exit(-1);
    }

    if ((ttf_file=open(argv[1], O_RDONLY, 0)) == -1) {
        fprintf(stderr, "**** Cannot open %s ****\n", argv[1]);
        exit(-1);
    } 

    if (read (ttf_file, filebuffer, statbuf.st_size) != statbuf.st_size) {
        fprintf(stderr, "**** Could not read whole file ****\n");
        exit(-1);
    }

    directory = (TTF_DIRECTORY *) filebuffer;

    if (ntohl(directory->sfntVersion) != 0x00010000) {
        fprintf(stderr,
                "****Unknown File Version number [%x], or not a TrueType file****\n",
                directory->sfntVersion);
        exit(-1);
    }


    dir_entry = &(directory->list);

    for (i=0; i < ntohs(directory->numTables); i++) {

        if (memcmp(dir_entry->tag, "name", 4) == 0) {
            name_table = (TTF_NAME *) (filebuffer+ntohl(dir_entry->offset));
        } 
        dir_entry++;
    }

    handle_name();

    fprintf(stdout, "/%s\t\t\t\t(%s)\t;\n",name_fields[6], argv[1]);

}



